/**
* Контроллер-исполнительное устройство к проекту IoT ioBroker
* Platform: Arduino UNO R3 + EthernetShield W5100
* IDE: Arduino 1.6.4
* Date: 18.12.2015
* Взаимодействие с ioBroker с помощью драйвера simple-api и скрипта (адаптер javascript)
* На цифровой Pin-9 подключена кнопка, цифровой Pin-8 - реле управления нагрузкой (свет)
* При изменении состояния кнопки (1->0 отпускание) происходит переключение сосотояния реле
* Так же состоянием реле можно управлять из адаптера VIS (отправка данных с помощью скрипта)
* При первоначальной загрузки контроллер делает запрос к ioBroker о сосотоянии реле и выставляет в соответствие порт Pin8.
* Состояние кнопки (true/false) и состоние реле (on/off) отображается в адаптере VIS.
*
* Обращение по http://xx.xx.xx.xx/ выдаст справочную информацию по этому устройству (нужно для того, чтобы
* по IP понять что это за контроллер и получить некоторую информацию по состоянию
*
**/

// Подключаем библиотеки
#include <Ethernet.h>
#include <SPI.h>

//Параметры сетевых коммуникаций
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xE4, 0xDE, 0x52 }; // MAC-адрес нашего устройства
byte ip[] = { 192, 168, 1, 252 }; // IP-адрес нашего устройства
byte rserver[] = { 192, 168, 1, 116 }; // IP-адрес сервера ioBroker

//Объявляем веб-сервер на порту 8081 и клиент для подключения к ioBroker
EthernetServer server(8081);
EthernetClient client;

//Буфер, содержащий строку для SET-запроса на сервер ioBroker
char buf[200];

//Объявляем переменные состояний портов
int DI = 0;
int DO = 0;
int DI_pin = 8;
int DO_pin = 9;
//Строка ответа от сервера ioBroker
String answer;
/**********************************************************************************************************************/
void setup()
{
  //Запускаем Ethernet соединение, сервер :
  Ethernet.begin(mac, ip);
  server.begin();
  //Запускаем консоль на порту USB ардуины для вывода отладочных сообщений
  Serial.begin(9600);
  //Выводим в консоль IP-адрес нашего устройства (значит Ethernet номально стартовал)
  Serial.println("Start controller Arduino1");
  Serial.print("Static IP is: ");
  Serial.println(Ethernet.localIP());
  //Объявляем порты
  pinMode (DI_pin, INPUT_PULLUP);
  pinMode (DO_pin, OUTPUT);
  //Проверяем начальное состояние портов дискретного ввода и отправляем на сервер
  Serial.println("Check DI state and send to server...");
  if (digitalRead (DI_pin)) {
    DI = true;
    sprintf(buf, "GET /set/javascript.0.Arduino1.DI?value=true");
    sendHTTPRequest();
  } else {
    DI = false;
    sprintf(buf, "GET /set/javascript.0.Arduino1.DI?value=false");
    sendHTTPRequest();
  }
  //Запрашиваем текущее состояние портов дискретного вывода на сервере и выставляем их в соответствие
  Serial.println("Check DO state in server, on/off on Arduino1");
  if (client.connect(rserver, 8087)) {
    Serial.println ("connect OK");
    Serial.println ("send GET /getPlainValue/javascript.0.Arduino1.DO");
    sprintf(buf, "GET /getPlainValue/javascript.0.Arduino1.DO");
    client.print(buf);
    client.println (" HTTP/1.0");
    client.println("Host: 192.168.1.252");
    client.print("Content-Type: text/html\n");
    client.println("Connection: close\n");
    delay(10);
    while(client.connected()) {
      while (client.available()) {
        char c = client.read();
	//Serial.write(c);
        answer = answer + c;
      }
    }
    client.stop();
    Serial.println ("");
    Serial.println ("--------------------");
    answer = answer.substring(answer.indexOf('"')+1,answer.lastIndexOf('"'));
    Serial.print(answer);
    Serial.println ("");
    Serial.println ("--------------------");
    if (answer == "on") digitalWrite (DO_pin, HIGH);
    if (answer == "off") digitalWrite (DO_pin, LOW);
    answer = "";
  } else {
    Serial.println("connect FAILED");
  }
}
/**********************************************************************************************************************/
void loop() {
//Обработка состояния DI (pin 8). При переходе с 1 в 0 (задний фронт) меняем состояние DO на противоположное.
//Отправляем каждое изменение DI на сервер
int tmp = digitalRead (DI_pin);
if (tmp != DI) {
  if (tmp == true && DI == false) {
    DI = tmp;
    sprintf(buf, "GET /set/javascript.0.Arduino1.DI?value=true");
    sendHTTPRequest();
  } else if (tmp == false && DI == true) {
    DI = tmp;
    sprintf(buf, "GET /set/javascript.0.Arduino1.DI?value=false");
    sendHTTPRequest();
    tmp = digitalRead (DO_pin);
    if (tmp) {
      digitalWrite (DO_pin, LOW);
      sprintf(buf, "GET /set/javascript.0.Arduino1.DO?value=off");
      sendHTTPRequest();
    } else {
      digitalWrite (DO_pin, HIGH);
      sprintf(buf, "GET /set/javascript.0.Arduino1.DO?value=on");
      sendHTTPRequest();
    }
  }
}
//Ожидаем входящих изменений от ioBroker (изменение состояния pin9)
//Либо запрос странички состояния от браузера
  EthernetClient now_client = server.available();
    if(now_client){
      String request;
      boolean currentLineIsBlank = true;
      boolean requestLineReceived = false;
      while(now_client.connected()){
        if (now_client.available()) {
          char c = now_client.read();
          Serial.write(c);
          if(c == '\n' && currentLineIsBlank) {
            // substract path from a request line
            request = request.substring(
              request.indexOf(' ') + 1,
              request.lastIndexOf(' '));
            if(request=="/"){
              successHeader(now_client);
              generatePageInfo(now_client);
            }
            else if(request=="/on"){
              successHeader(now_client);
              digitalWrite (DO_pin, HIGH);
            }
            else if(request=="/off"){
              successHeader(now_client);
              digitalWrite (DO_pin, LOW);
            }
            break;
          }
          if (c == '\n') {
            currentLineIsBlank = true;
            if(!requestLineReceived){
              requestLineReceived = true;
            }
          }
          else if (c != '\r') {
            if(!requestLineReceived) {
              request += c;
            }
            currentLineIsBlank = false;
          }
        }
      }
    delay(10);
    now_client.stop();
  }
}

/**********************************************************************************************************************/
//Функция генерирует заголовок http ответа с кодом 200 - успешное выполнение запроса
void successHeader(EthernetClient now_client){
  now_client.println("HTTP/1.1 200 OK");
  now_client.println("Content-Type: text/html");
  now_client.println("Connection: close");
  now_client.println();
}
//Генерация html-кода информационной странички "Об устройстве"
void generatePageInfo(EthernetClient now_client){
  DI = digitalRead (DI_pin);
  DO = digitalRead (DO_pin);
  now_client.println("<!DOCTYPE HTML>");
  now_client.println("<html>");
  //now_client.println("<meta http-equiv=\"refresh\" content=\"5\">");
  now_client.println("<head>");
  now_client.println("<title>Arduino WEB server</title>");
  now_client.println("</head>");
  now_client.println("<body>");
  now_client.println("<h3>Arduino WEB server for CCU.IO</h3>");
  now_client.println("Arduino UNO board and ethernet shield W5100<br>");
  now_client.println("<br>Current status DI, DO:");
  now_client.print("DI=");
  now_client.print(DI);
  now_client.print(" DO=");
  now_client.println(DO);
  now_client.println("</body>");
  now_client.println("</html>");
}
// Функция отправки HTTP-запроса на сервер
void sendHTTPRequest() {
  if (client.connect(rserver, 8087)) {
    Serial.println ("connect OK");    
    Serial.println(buf);
    client.print(buf);
    client.println (" HTTP/1.0");
    client.println("Host: 192.168.1.252");
    client.print("Content-Type: text/html\n");
    client.println("Connection: close\n");
    delay(10);
    client.stop();
    Serial.println ("--------------------");
  } else {
    Serial.println("connect FAILED");
    }
}
